@@ -6,7 +6,8 @@ from security_manager_apis.get_properties_data import get_properties_data
 
 class PolicyPlannerApis():
 
-    def __init__(self, host: str, username: str, password: str, verify_ssl: bool, domain_id: str, workflow_name: str, suppress_ssl_warning=False):
+    def __init__(self, host: str, username: str, password: str, verify_ssl: bool, domain_id: str, workflow_name: str,
+                 suppress_ssl_warning=False):
         """ User needs to pass host,username,password,and verify_ssl as parameters while
             creating instance of this class and internally Authentication class instance
             will be created which will set authentication token in the header to get firemon API access
@@ -21,14 +22,15 @@ class PolicyPlannerApis():
         self.api_resp = ''
         self.domain_id = domain_id
         self.workflow_id = self.get_workflow_id_by_workflow_name(domain_id, workflow_name)
-        
+
     def create_pp_ticket(self, request_body: dict) -> dict:
         """
         making call to create pp ticket api which creates a policy planner ticket on corresponding FMOS box
         :param request_body: JSON body for ticket.
         :return: JSON of ticket
         """
-        pp_tkt_url = self.parser.get('REST', 'create_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id)
+        pp_tkt_url = self.parser.get('REST', 'create_pp_tkt_api_url').format(self.host, self.domain_id,
+                                                                             self.workflow_id)
         try:
             resp = requests.post(url=pp_tkt_url,
                                  headers=self.headers, json=request_body, verify=self.verify_ssl)
@@ -44,15 +46,15 @@ class PolicyPlannerApis():
         :return: JSON of results
         """
         pp_tkt_url = self.parser.get('REST', 'siql_query_pp_tkt_api').format(self.host, self.domain_id)
-        parameters = {'q': siql_query, 'pageSize': page_size, 'domainid': self.domain_id }
+        parameters = {'q': siql_query, 'pageSize': page_size, 'domainid': self.domain_id}
         try:
             resp = requests.get(url=pp_tkt_url,
-                                 headers=self.headers, params=parameters, verify=self.verify_ssl)
+                                headers=self.headers, params=parameters, verify=self.verify_ssl)
             return resp.json()
         except requests.exceptions.HTTPError as e:
             print("Exception occurred while querying policy planner tickets with workflow id '{0}'\n Exception : {1}".
                   format(workflow_id, e.response.text))
-    
+
     def update_pp_ticket(self, ticket_id: str, request_body: dict) -> str:
         """
         Updates ticket in Policy Planner.
@@ -60,40 +62,44 @@ class PolicyPlannerApis():
         :param ticket_id: Ticket ID
         :return: Status code of API Call
         """
-        pp_tkt_url = self.parser.get('REST', 'update_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'update_pp_tkt_api_url').format(self.host, self.domain_id,
+                                                                             self.workflow_id, ticket_id)
         try:
             resp = requests.put(url=pp_tkt_url,
-                                 headers=self.headers, json=request_body, verify=self.verify_ssl)
+                                headers=self.headers, json=request_body, verify=self.verify_ssl)
             return str(resp.status_code)
         except requests.exceptions.HTTPError as e:
             print("Exception occurred while creating policy planner ticket with workflow id '{0}'\n Exception : {1}".
                   format(workflow_id, e.response.text))
-    
+
     def pull_pp_ticket(self, ticket_id: str) -> dict:
         """
         making call to retrieve pp ticket api which retrieves a policy planner ticket on corresponding FMOS box
         :param ticket_id: ID of ticket
         :return: JSON of ticket
         """
-        pp_tkt_url = self.parser.get('REST', 'pull_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'pull_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id,
+                                                                           ticket_id)
         try:
             resp = requests.get(url=pp_tkt_url,
-                                 headers=self.headers, verify=self.verify_ssl)
+                                headers=self.headers, verify=self.verify_ssl)
             return resp.json()
         except requests.exceptions.HTTPError as e:
             print("Exception occurred while retrieving policy planner ticket with workflow id '{0}'\n Exception : {1}".
                   format(workflow_id, e.response.text))
-    
+
     def assign_pp_ticket(self, ticket_id: str, user_id: str) -> str:
         """ making call to assign pp ticket api which
             asigns a policy planner ticket on corresponding FMOS box """
         ticket_json = self.pull_pp_ticket(ticket_id)
         workflow_packet_task_id = self.get_workflow_packet_task_id(ticket_json)
         workflow_task_id = self.get_workflow_task_id(ticket_json)
-        pp_tkt_url = self.parser.get('REST', 'assign_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id, workflow_task_id, ticket_id, workflow_packet_task_id)
+        pp_tkt_url = self.parser.get('REST', 'assign_pp_tkt_api_url').format(self.host, self.domain_id,
+                                                                             self.workflow_id, workflow_task_id,
+                                                                             ticket_id, workflow_packet_task_id)
         try:
             resp = requests.put(url=pp_tkt_url,
-                                 headers=self.headers, data=user_id, verify=self.verify_ssl)
+                                headers=self.headers, data=user_id, verify=self.verify_ssl)
             return str(resp.status_code)
         except requests.exceptions.HTTPError as e:
             print("Exception occurred while assigning policy planner ticket with workflow id '{0}'\n Exception : {1}".
@@ -102,15 +108,17 @@ class PolicyPlannerApis():
     def add_req_pp_ticket(self, ticket_id: str, req_json: dict) -> str:
         ticket_json = self.pull_pp_ticket(ticket_id)
         workflow_task_id = self.get_workflow_task_id(ticket_json)
-        pp_tkt_url = self.parser.get('REST', 'add_req_pp_tkt_api_url').format(self.host, self.domain_id, self.workflow_id,
-                                                                             workflow_task_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'add_req_pp_tkt_api_url').format(self.host, self.domain_id,
+                                                                              self.workflow_id,
+                                                                              workflow_task_id, ticket_id)
         try:
             resp = requests.post(url=pp_tkt_url,
                                  headers=self.headers, json=req_json, verify=self.verify_ssl)
             return str(resp.status_code)
         except requests.exceptions.HTTPError as e:
-            print("Exception occurred while adding a requirement to policy planner ticket with workflow id '{0}'\n Exception : {1}".
-                  format(workflow_id, e.response.text))
+            print(
+                "Exception occurred while adding a requirement to policy planner ticket with workflow id '{0}'\n Exception : {1}".
+                format(workflow_id, e.response.text))
 
     def complete_task_pp_ticket(self, ticket_id: str, button_action: str) -> list:
         """
@@ -122,14 +130,16 @@ class PolicyPlannerApis():
         workflow_packet_task_id = self.get_workflow_packet_task_id(ticket_json)
         workflow_task_id = self.get_workflow_task_id(ticket_json)
         pp_tkt_url = self.parser.get('REST', 'comp_task_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id,
-                                                                             workflow_task_id, ticket_id, workflow_packet_task_id, button_action)
+                                                                            workflow_task_id, ticket_id,
+                                                                            workflow_packet_task_id, button_action)
         try:
             resp = requests.put(url=pp_tkt_url,
-                                 headers=self.headers, json={}, verify=self.verify_ssl)
+                                headers=self.headers, json={}, verify=self.verify_ssl)
             return resp.status_code, resp.reason
         except requests.exceptions.HTTPError as e:
-            print("Exception occurred while completing task on policy planner ticket with workflow id '{0}'\n Exception : {1}".
-                  format(workflow_id, e.response.text))
+            print(
+                "Exception occurred while completing task on policy planner ticket with workflow id '{0}'\n Exception : {1}".
+                format(workflow_id, e.response.text))
 
     def do_pca(self, ticket_id: str, control_types: str, enable_risk_sa: str) -> list:
         """
@@ -143,16 +153,16 @@ class PolicyPlannerApis():
         """
         controls_formatted = self.parse_controls(control_types)
         pp_tkt_url = self.parser.get('REST', 'run_pca_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id,
-                                                                            ticket_id, controls_formatted,
-                                                                            enable_risk_sa)
+                                                                          ticket_id, controls_formatted,
+                                                                          enable_risk_sa)
         try:
             resp = requests.post(url=pp_tkt_url,
-                                headers=self.headers, verify=self.verify_ssl)
+                                 headers=self.headers, verify=self.verify_ssl)
             return resp.status_code, resp.reason
         except requests.exceptions.HTTPError as e:
             print(
                 "Exception occurred while running PCA on policy planner ticket with workflow id '{0}'\n Exception : {1}".
-                format(workflow_id, e.response.text))
+                    format(workflow_id, e.response.text))
 
     def retrieve_pca(self, ticket_id: str) -> dict:
         """
@@ -195,7 +205,7 @@ class PolicyPlannerApis():
                 output = output + '&'
             output = output + 'controlTypes=' + controls_list[c]
         return output
-    
+
     def stage_attachment(self, file_name: str, f) -> str:
         pp_tkt_url = self.parser.get('REST', 'stage_att_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id)
         new_headers = self.headers
@@ -211,14 +221,16 @@ class PolicyPlannerApis():
     def post_attachment(self, ticket_id: str, attachment_json: dict) -> dict:
         new_headers = self.headers
         new_headers.pop('Content-Type', None)
-        pp_tkt_url = self.parser.get('REST', 'post_att_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'post_att_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id,
+                                                                           ticket_id)
         try:
             resp = requests.put(url=pp_tkt_url,
-                                 headers=new_headers, json=attachment_json, verify=self.verify_ssl)
+                                headers=new_headers, json=attachment_json, verify=self.verify_ssl)
             return resp.json()
         except requests.exceptions.HTTPError as e:
-            print("Exception occurred while adding attachment to policy planner ticket with workflow id '{0}'\n Exception : {1}".
-                  format(workflow_id, e.response.text))
+            print(
+                "Exception occurred while adding attachment to policy planner ticket with workflow id '{0}'\n Exception : {1}".
+                format(workflow_id, e.response.text))
 
     def add_attachment(self, ticket_id: str, file_name: str, f, description: str):
         attachment_staged = self.stage_attachment(file_name, f)
@@ -254,8 +266,9 @@ class PolicyPlannerApis():
         """
         ticket_json = self.pull_pp_ticket(ticket_id)
         workflow_task_id = self.get_workflow_task_id(ticket_json)
-        pp_tkt_url = self.parser.get('REST', 'get_recs_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, workflow_task_id,
-                                                                          ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'get_recs_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id,
+                                                                           workflow_task_id,
+                                                                           ticket_id)
         try:
             resp = requests.get(url=pp_tkt_url,
                                 headers=self.headers, verify=self.verify_ssl)
@@ -281,7 +294,7 @@ class PolicyPlannerApis():
                                                                                ticket_id, str(r['id']))
             try:
                 resp = requests.delete(url=pp_tkt_url,
-                                    headers=self.headers, verify=self.verify_ssl)
+                                       headers=self.headers, verify=self.verify_ssl)
             except requests.exceptions.HTTPError as e:
                 print(
                     "Exception occurred while deleting requirements on policy planner ticket with workflow id '{0}'\n Exception : {1}".
@@ -290,25 +303,48 @@ class PolicyPlannerApis():
         return reqs
 
     def approve_req(self, ticket_id: str, req_id: str) -> list:
-        pp_tkt_url = self.parser.get('REST', 'app_req_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, ticket_id, req_id)
-        print(pp_tkt_url)
+        pp_tkt_url = self.parser.get('REST', 'app_req_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id,
+                                                                          ticket_id, req_id)
         try:
             resp = requests.put(url=pp_tkt_url,
-                                 headers=self.headers, json={}, verify=self.verify_ssl)
+                                headers=self.headers, json={}, verify=self.verify_ssl)
             return resp.status_code, resp.reason
         except requests.exceptions.HTTPError as e:
             print(
                 "Exception occurred while approving requirement on policy planner ticket with workflow id '{0}'\n Exception : {1}".
                     format(workflow_id, e.response.text))
 
+    def add_change(self, ticket_id: str, req_id: str, change: dict) -> list:
+        """
+        Add change to policy planner requirement
+        :param ticket_id: ID of ticket
+        :param req_id: ID of requirement
+        :param change: JSON of change
+        :return: Response code, reason, JSON as list
+        """
+        ticket_json = self.pull_pp_ticket(ticket_id)
+        workflow_task_id = self.get_workflow_task_id(ticket_json)
+        pp_tkt_url = self.parser.get('REST', 'add_change_pp_tkt_api').format(self.host, self.domain_id,
+                                                                              self.workflow_id, workflow_task_id,
+                                                                              ticket_id, req_id)
+        try:
+            resp = requests.post(url=pp_tkt_url,
+                                headers=self.headers, json=change, verify=self.verify_ssl)
+            return resp.status_code, resp.reason, resp.json()
+        except requests.exceptions.HTTPError as e:
+            print(
+                "Exception occurred while approving requirement on policy planner ticket with workflow id '{0}'\n Exception : {1}".
+                    format(workflow_id, e.response.text))
+
     def add_comment(self, ticket_id: str, comment: str) -> list:
         comment_json = {
             'comment': comment
         }
-        pp_tkt_url = self.parser.get('REST', 'add_comment_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'add_comment_pp_tkt_api').format(self.host, self.domain_id,
+                                                                              self.workflow_id, ticket_id)
         try:
             resp = requests.post(url=pp_tkt_url,
-                                headers=self.headers, json=comment_json,verify=self.verify_ssl)
+                                 headers=self.headers, json=comment_json, verify=self.verify_ssl)
             return resp.status_code, resp.reason
         except requests.exceptions.HTTPError as e:
             print(
@@ -316,10 +352,11 @@ class PolicyPlannerApis():
                     format(workflow_id, e.response.text))
 
     def get_comments(self, ticket_id: str) -> dict:
-        pp_tkt_url = self.parser.get('REST', 'get_comments_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, ticket_id)
+        pp_tkt_url = self.parser.get('REST', 'get_comments_pp_tkt_api').format(self.host, self.domain_id,
+                                                                               self.workflow_id, ticket_id)
         try:
             resp = requests.get(url=pp_tkt_url,
-                                headers=self.headers,verify=self.verify_ssl)
+                                headers=self.headers, verify=self.verify_ssl)
             return resp.json()
         except requests.exceptions.HTTPError as e:
             print(
@@ -327,16 +364,17 @@ class PolicyPlannerApis():
                     format(workflow_id, e.response.text))
 
     def del_comment(self, ticket_id: str, comment_id: str) -> list:
-        pp_tkt_url = self.parser.get('REST', 'del_comment_pp_tkt_api').format(self.host, self.domain_id, self.workflow_id, ticket_id, comment_id)
+        pp_tkt_url = self.parser.get('REST', 'del_comment_pp_tkt_api').format(self.host, self.domain_id,
+                                                                              self.workflow_id, ticket_id, comment_id)
         try:
             resp = requests.delete(url=pp_tkt_url,
-                                headers=self.headers,verify=self.verify_ssl)
+                                   headers=self.headers, verify=self.verify_ssl)
             return resp.status_code, resp.reason
         except requests.exceptions.HTTPError as e:
             print(
                 "Exception occurred while deleting comment on policy planner ticket with workflow id '{0}'\n Exception : {1}".
                     format(workflow_id, e.response.text))
-    
+
     def get_workflow_packet_task_id(self, ticket_json: dict) -> str:
         """
         Retrieves workflowPacketTaskId value from current stage of provided ticket
@@ -364,7 +402,6 @@ class PolicyPlannerApis():
     def get_workflow_id_by_workflow_name(self, domain_id: str, workflow_name: str) -> str:
         """ Takes domainId and workflow name as input parameters and returns you
             the workflowId for given workflow name """
-
         workflow_url = self.parser.get('REST', 'find_all_workflows_url').format(self.host, domain_id)
         try:
 
